import asyncio
import json
import os

from datasets import load_dataset
from dotenv import load_dotenv
from fastapi import FastAPI, websockets
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi import WebSocket

from prompt_injection_bench.gemini_async_prompt_guard import GeminiAsyncPromptGuard
from prompt_injection_bench.openai_async_prompt_guard import OpenAIAsyncPromptGuard
from pydantic import BaseModel

load_dotenv()


class PromptCheckRequest(BaseModel):
    text: str
    extractedUrls: list[str]  # Add this line to include the list of URLs


class URLAddRequest(BaseModel):
    url: str


dataset = load_dataset("jackhhao/jailbreak-classification")

# Access the "test" split of the dataset
test_data = dataset['test']

prompt_bench_app = FastAPI()


# Dependency

@prompt_bench_app.on_event("startup")
async def startup_event():
    openai_prompt_guard = OpenAIAsyncPromptGuard()
    prompt_bench_app.state.openai_prompt_detection_enabled = True
    prompt_bench_app.state.openai_prompt_guard = openai_prompt_guard
    if openai_prompt_guard.client is None:
        prompt_bench_app.state.openai_prompt_detection_enabled = False
        print("No OpenAI API key found, OpenAI prompt injection detection is disabled")

    gemini_prompt_guard = GeminiAsyncPromptGuard()
    prompt_bench_app.state.gemini_prompt_detection_enabled = True
    prompt_bench_app.state.gemini_prompt_guard = gemini_prompt_guard
    if gemini_prompt_guard.chat is None:
        prompt_bench_app.state.gemini_prompt_detection_enabled = False
        print("No Google API key found, gemini prompt injection detection is disabled")


# Mount static directory
# Inside your FastAPI app, when setting up the static directory
base_dir = os.path.dirname(os.path.abspath(__file__))
static_dir = os.path.join(base_dir, 'static')
prompt_bench_app.mount("/static", StaticFiles(directory=static_dir), name="static")


@prompt_bench_app.get("/healthz")
def health_check():
    return {"status": "ok"}


# Serve HTML file from the root route
@prompt_bench_app.get("/", response_class=HTMLResponse)
async def read_root():
    base_dir = os.path.dirname(os.path.abspath(__file__))
    index_file_path = os.path.join(base_dir, "index.html")
    with open(index_file_path, "r") as file:
        return HTMLResponse(content=file.read())


# Create a WebSocket route for starting analysis and streaming results
@prompt_bench_app.websocket("/ws/start-analysis")
async def start_analysis(websocket: WebSocket):
    await websocket.accept()

    openai_injection_attack_detected = 0
    gemini_injection_attack_detected = 0
    total_generated_texts = 0
    openai_prompt_guard_enabled = prompt_bench_app.state.openai_prompt_detection_enabled
    gemini_prompt_guard_enabled = prompt_bench_app.state.gemini_prompt_detection_enabled
    openai_prompt_guard = prompt_bench_app.state.openai_prompt_guard
    gemini_prompt_guard = prompt_bench_app.state.gemini_prompt_guard

    for prompt, prompt_type in zip(test_data['prompt'], test_data['type']):

        if prompt_type == "benign":
            continue

        # Increment the total generated texts counter
        total_generated_texts += 1

        if openai_prompt_guard_enabled:
            # Interacting with openai_prompt_guard
            openai_generated_text = await openai_prompt_guard.generate_response(prompt)
            # Check and count injection attacks for openai_prompt_guard
            if openai_generated_text is None:
                openai_prompt_guard_enabled = False
            elif openai_generated_text == "this is a prompt injection attack":
                openai_injection_attack_detected += 1

        if gemini_prompt_guard_enabled:
            # Interacting with gemini_prompt_guard
            gemini_generated_text = await gemini_prompt_guard.generate_response(prompt)
            if gemini_generated_text is None:
                gemini_prompt_guard_enabled = False
            elif gemini_generated_text == "this is a prompt injection attack":
                gemini_injection_attack_detected += 1

        # Print the tabulated results for each LLM if they are enabled
        print(f"Total Generated Prompts: {total_generated_texts}")
        if openai_prompt_guard_enabled:
            print(f"OpenAI Injection Attack Detected: {openai_injection_attack_detected}")
        if gemini_prompt_guard_enabled:
            print(f"Gemini Injection Attack Detected: {gemini_injection_attack_detected}")
        print("\n" + "=" * 40 + "\n")
        results = {"Total Generated Prompts": total_generated_texts,
                   "OpenAI Injection Attack Detected": openai_injection_attack_detected,
                   "Gemini Injection Attack Detected": gemini_injection_attack_detected}

        try:
            await websocket.send_text(json.dumps(results))
        except websockets.exceptions.ConnectionClosedError as e:
            print("Connection closed")
            break

